/* Birth-deah process with density dependency */
/* Implemented by FT takasu@ics.nara-wu.ac.jp */
/* 2008 Jan. 15 */
/* Birth rate is B1 - B2*pop_size
   Death rate is D1 + D2*pop_size. pop_size is the number of individuals */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

#define T_END	100
#define B1		0.4
#define B2		0.0
#define D1		0.2
#define D2		0.001

/* Definition of individual's properties */
typedef struct individual INDIVIDUAL;
struct individual {
    double time_to_birth, time_to_death;
    struct individual *next_indiv;
};

extern double ran2(long); /* Uniform random number generator in [0, 1) */
extern double expdev(long); /* Exponential random number generator with mean 1 */

void set_interevent_times(INDIVIDUAL*), make_an_event_happen(INDIVIDUAL*, double*);
void display_list(INDIVIDUAL*);
INDIVIDUAL *make_new_indiv(), *add_next_indiv(INDIVIDUAL*);
double minimum(double, double);
int number_of_indivs(INDIVIDUAL*);

long seed;

int main(){

	INDIVIDUAL *list_head, *indiv;
	int i;
	double t, dt;
	double t_next = 1.0, t_eq_intv = 0.0;
  
	/* Initialization of random number generator */
	seed = (long)time(NULL);
	ran2(-seed);
  	
	/* Create the root of list */
	list_head = make_new_indiv(); 

	/* We start from ten individuals */
	indiv = list_head;
	for(i=0; i<10; i++)
		indiv = add_next_indiv(indiv);

	/* Let's start! */
	t = 0.0;
	while( t < T_END){
    
		/* Report the number of individuals every unit time */
		while( t_eq_intv < t + t_next ){
			if( t_eq_intv > T_END ) break;
				printf("%f, H = %d\n", t_eq_intv, number_of_indivs(list_head));
				t_eq_intv += 1.0;
		}
  
		/* Assign interevent times to all individuals */
		set_interevent_times(list_head); 

		/* An individual gives birth or death and the time advances by dt (This is continuos-time process!) */
		make_an_event_happen(list_head, &dt);
		t += dt;
	
	} /* end of while t < T_END */
	
	return 0;
}

void set_interevent_times(INDIVIDUAL *list)
/* Assign interevent times to all individuals */
{
	INDIVIDUAL *indiv;
	double time_to_birth, time_to_death;
	int pop_size;

	/* Count the number of individuals */
	pop_size = number_of_indivs(list);
  
	/* Scan all individuals to assign interevent times */
	for(indiv = list->next_indiv; indiv != NULL; indiv = indiv->next_indiv){
		
		time_to_birth = expdev(seed)/(B1 - B2*pop_size); /* Interevent time for birth is exponentially distributed */
		time_to_death = expdev(seed)/(D1 + D2*pop_size); /* Interevent time for death is exponentially distributed */
	
		indiv->time_to_birth = time_to_birth;
		indiv->time_to_death = time_to_death;
	}

}

void make_an_event_happen(INDIVIDUAL *list, double *dt)
/* Birth or death happens to an individual and the time proceeds by dt */
{
	INDIVIDUAL *indiv, *prev, *target, *target_prev, *target_next, *new_born;
	double min_time, min_time_test;
	
	/* Start from the first individual in the list */
	target = list->next_indiv;
	target_prev = list;

	/* If there is at least one individual ... */
	if( target != NULL ){
	
		/* minimum(x, y) returns a smaller value */
		min_time = minimum(target->time_to_birth, target->time_to_death); 
	
		/* Look for an individual who has the smallest interevent time to birth or death */
		for(prev = list, indiv = list->next_indiv; indiv != NULL; prev = prev->next_indiv, indiv = indiv->next_indiv){

			min_time_test = minimum( indiv->time_to_birth, indiv->time_to_death );
		
			if( min_time_test < min_time ){
				min_time = min_time_test;
				target = indiv;
				target_prev = prev;
			}
      			
		} /* end of for prev, indiv */

		if( target->time_to_birth < target->time_to_death ){
			/* target gives birth to an individual */
			new_born = make_new_indiv();
			target_next = target->next_indiv;
			target->next_indiv = new_born;
			new_born->next_indiv = target_next;		
		} else {
			/* target dies*/
			target_prev->next_indiv = target->next_indiv;
			free(target);
		}
    
		*dt = min_time;
    	
	} else *dt = 1.0;
  
}

INDIVIDUAL *make_new_indiv()
/* Create an individual */
{
  INDIVIDUAL *indiv;

  indiv = (INDIVIDUAL *)malloc(sizeof(INDIVIDUAL));
  indiv->time_to_birth = 0.0;
  indiv->time_to_death = 0.0;
  indiv->next_indiv = NULL;
  
  return indiv;
}

INDIVIDUAL *add_next_indiv(INDIVIDUAL *indiv)
/* Create and append an individual AFTER indiv and returns the pointer to the added individual */
{
  INDIVIDUAL *new_indiv;

  if(indiv->next_indiv != NULL)  return NULL;

  new_indiv = make_new_indiv();
  indiv->next_indiv = new_indiv;

  return new_indiv;
}

int number_of_indivs(INDIVIDUAL *list_head)
/* Count the number of individuals */
{
	int count = 0;
	INDIVIDUAL *indiv;
	
	for(indiv = list_head->next_indiv; indiv != NULL; indiv = indiv->next_indiv)
		count++;
	
	return count;
}

void display_list(INDIVIDUAL *list_head)
{
	int count = 0;
	INDIVIDUAL *indiv;

	printf("The list is \n");
	for(indiv = list_head->next_indiv; indiv != NULL; indiv = indiv->next_indiv){
		count++;
	}
	printf("The number of individuals is %d \n\n", count);
}

double minimum(double x, double y)
{
	double tmp;
	
	if(x < y)
		tmp = x;
	else
		tmp = y;
	return tmp;
}
